@@ -1,201 +1,34 @@
 '''astro_reduce -- A Simple CCD Images Reducer for the Paris Observatory.'''
 
-from sys import exit
-from os.path import basename, exists, getsize
-from os import mkdir, getcwd
-from shutil import copy, rmtree
-from glob import glob
-from json import loads, dump, decoder
+
 from collections import defaultdict
-from re import compile, sub
-from hashlib import md5
-from time import time
 from datetime import timedelta
+from glob import glob
+from json import loads, decoder
+from os.path import basename, exists, getsize
+from os import mkdir, getcwd, remove
+from re import compile, sub
+from shutil import copy, rmtree
+from sys import exit
+from time import time
 
 import click
 import numpy as np
-import matplotlib.pyplot as plt
-import matplotlib.colors as colors
-from astropy.io import fits
-from astropy.visualization import ImageNormalize, ZScaleInterval
-from scipy.signal import fftconvolve
+from matplotlib import pyplot as plt
 
-
-# Comment for header keywords.
-hc = 'Exposure time in seconds'
-
-# Paths and extensions.
-# User image directories.
-UDARK = 'DARK'
-UFLAT = 'FLAT'
-UOBJ = 'ORIGINAL'
-
-# Astro-reduce working directories.
-OBJ = 'ar_objects'
-DARK = 'ar_darks'
-FLAT = 'ar_flats'
-TMP = 'tmp'
-
-# File names and extensions.
-di = 'dark'
-fi = 'flat'
-RED = 'reduced'
-AUX = 'aux'
-
-# Simple hashing function for file names.
-hsh = lambda x: md5(x.encode('utf-8')).hexdigest()
-
-
-def align_and_median(infiles):
-    '''Read fits data from list of files, return aligned and medianed version.
-    '''
-    if len(infiles) == 1:
-        return fits.getdata(infiles[0])
-
-    # Collect arrays and crosscorrelate all (except the first) with the first.
-    images = [fits.getdata(_).astype(np.float64) for _ in infiles]
-    nX, nY = images[0].shape
-    correlations = [fftconvolve(images[0], image[::-1, ::-1], mode='same')
-                    for image in images[1:]]
-
-    # For each image determine the coordinate of maximum cross-correlation.
-    shift_indices = [np.unravel_index(np.argmax(corr_array, axis=None),
-                                      corr_array.shape)
-                     for corr_array in correlations]
-    deltas = [(ind[0] - int(nX / 2), ind[1] - int(nY / 2))
-              for ind in shift_indices]
-
-    # Warn for ghost images if realignment requires shifting by more than
-    # 15% of the field size.
-    if (abs(max(deltas, key=lambda x: abs(x[0]))[0]) > nX * 0.15
-       or abs(max(deltas, key=lambda x: abs(x[1]))[1]) > nY * 0.15):
-        pieces = basename(infiles[0]).split('_')
-        click.echo('W: In {}:{}:{}, shifting by more than 15% of the field. '
-                   'Beware of ghosts!'.format(pieces[0], pieces[1], pieces[2]))
-
-    # Roll the images to realign them and return their median.
-    realigned_images = [np.roll(image, deltas[i], axis=(0, 1))
-                        for (i, image) in enumerate(images[1:])]
-    realigned_images.append(images[0])
-    return np.median(realigned_images, axis=0)
-
-
-def dark_read_header(fname):
-    '''Return exposure and standard file name for a dark field image.'''
-    head = fits.getheader(fname)
-    if 'EXPTIME' in head.keys():
-        exp = int(1000 * head['EXPTIME'])
-    elif 'EXPOSURE' in head.keys():
-        exp = int(1000 * head['EXPOSURE'])
-    elif 'EXP (MS)' in head.keys():
-        exp = int(head['EXP (MS)'])
-    else:
-        raise IOError('No exposure keyword in header of `{}`.'.format(fname))
-    return exp, '{}_{}_{}.fits'.format(di, exp, hsh(fname))
-
-
-def flat_read_header(fname):
-    '''Return filter, exposure, standard file name for a flat field image.'''
-    head = fits.getheader(fname)
-    # Filter.
-    if 'FILTER' in head.keys():
-        fil = sub('[- _]', '', head['FILTER'])
-    else:
-        raise IOError('No filter keyword in header of `{}`.'.format(fname))
-
-    # Exposure.
-    if 'EXPTIME' in head.keys():
-        exp = int(1000 * head['EXPTIME'])
-    elif 'EXPOSURE' in head.keys():
-        exp = int(1000 * head['EXPOSURE'])
-    elif 'EXP (MS)' in head.keys():
-        exp = int(head['EXP (MS)'])
-    else:
-        raise IOError('No exposure keyword in header of `{}`.'.format(fname))
-
-    return fil, exp, '{}_{}_{}_{}.fits'.format(fi, fil, exp, hsh(fname))
-
-
-def obj_read_header(fname):
-    '''Return object, filter, exposure and standard file name for object image.
-    '''
-    head = fits.getheader(fname)
-    # Object.
-    if 'OBJECT' in head.keys():
-        obj = sub('[ _]', '-', head['OBJECT'])
-    else:
-        raise IOError('No object keyword in header of `{}`.'.format(fname))
-
-    # Filter.
-    if 'FILTER' in head.keys():
-        fil = sub('[- _]', '', head['FILTER'])
-    else:
-        raise IOError('No filter keyword in header of `{}`.'.format(fname))
-
-    # Exposure.
-    if 'EXPTIME' in head.keys():
-        exp = int(1000 * head['EXPTIME'])
-    elif 'EXPOSURE' in head.keys():
-        exp = int(1000 * head['EXPOSURE'])
-    elif 'EXP (MS)' in head.keys():
-        exp = int(head['EXP (MS)'])
-    else:
-        raise IOError('No exposure keyword in header of `{}`.'.format(fname))
-
-    return obj, fil, exp, '{}_{}_{}_{}.fits'.format(obj, fil, exp, hsh(fname))
-
-
-def write_conf_file(objects, exposures, filters, conf_file_name):
-    '''Write configuration file from list of objects, exposures, filters.'''
-    conf_dic = {'objects': objects,
-                'exposures': exposures,
-                'filters': filters}
-    with open(conf_file_name, 'w') as cdfile:
-        dump(conf_dic, cdfile, indent=2)
-
-
-def fname_bits(fname):
-    '''Return the filter and exposure from standard file name of an object.
-
-    'NGC1000_V_1000_0.fits' gives ('V', '1000').
-    '''
-    pieces = fname.split('.fit')[0].split('_')
-    return (pieces[-3], pieces[-2])
-
-
-def write_png(fname, plt):
-    '''Generate PNG version of image read in a fits file.
-
-    Try to use a zscale normalization, and fall back to a classical
-    linear scale between the min and max of 1000 randomly picked pixels of the
-    image. Save the PNG image in same directory as the fits file.
-    '''
-    image = fits.getdata(fname)
-    norm = ImageNormalize(image, ZScaleInterval())
-    plt.figure(42)
-    plt.imshow(image, norm=norm, cmap='jet')
-    try:
-        # If the zscale algorithm doesn't converge, an UnboundLocalError is
-        # raised by astropy.visualization ...
-        plt.colorbar()
-    except UnboundLocalError:
-        # ... in this case, just pick 1000 random pixels and linearly scale
-        # between them.
-        plt.clf()
-        sample = np.random.choice(image.ravel(), 1000)
-        norm = colors.Normalize(np.min(sample), np.max(sample), clip=True)
-        plt.imshow(image, norm=norm, cmap='jet')
-        plt.colorbar()
-    plt.title(basename(fname).split(".fit")[0])
-    plt.savefig('{}.png'.format(fname.split(".fit")[0]), bbox_inches='tight')
-    plt.close(42)
+from cosmetic import align_and_median
+from helpers import *
 
 
 @click.command()
 @click.version_option()
 @click.option('--setup', '-s', is_flag=True,
-              help='Sets up the directory for reduction. Use this option only '
-              'the first time astro_reduce is run in the directory.')
+              help='Set up the directory for reduction. Use this option the '
+              'first time astro_reduce is run in the directory or after the '
+              '--clear option was used.')
+@click.option('--clear', '-c', is_flag=True,
+              help='Remove all astro_reduce-related files and folders in '
+              'current directory and exit.')
 @click.option('--interpolate', '-i', is_flag=True,
               help='Interpolate existing dark fields if some are missing.')
 @click.option('--verbose', '-v', is_flag=True,
@@ -204,71 +37,94 @@ def write_png(fname, plt):
               help='Write PNG format of intermediate images after reduction.')
 @click.option('--redpng', '-r', is_flag=True,
               help='Write PNG format of reduced images after reduction.')
-def cli(setup, interpolate, verbose, tmppng, redpng):
+def cli(setup, clear, interpolate, verbose, tmppng, redpng):
     '''Main run of astro_reduce:
 
-    i) Copy all user fits data to files with standard names in the working
-       directories.
-    ii) Reduce dark field images to master darks for all exposures.
-    iii) If necessary, interpolate darks for exposures lacking dark fields.
-    iv) Reduce all flat field images to master transmission files.
-    v) Reduce all object images with the master reduction files.
-    vi) Realign the object images of same series.
-    vii) Generate PNG versions of temporary and reduced images.
+    If --setup option is on: copy all user fits data to files with standard
+       names in the working directories, and exit.
+
+    If --clear option is on: clear directory of all astro_reduce working data
+    and exit.
+
+    Else, reduction procedure:
+    i) Reduce dark field images to master darks for all exposures.
+    ii) If necessary, interpolate darks for exposures lacking dark fields.
+    iii) Reduce all flat field images to master transmission files.
+    iv) Reduce all object images with the master reduction files.
+    v) Realign the object images of same series.
+
+    If --{tmp,red}png options are on: generate PNG versions of temporary
+    and reduced images.
 
     '''
+    # Current working directory.
+    cwd = basename(getcwd())
+
+    # Welcome.
+    click.secho('    Welcome to astro_reduce!\n'
+                '    Software is copyright 2018-2021 R. Duque.\n\n',
+                fg='cyan', bold=True)
+    click.secho('Currently working in directory `{}`.'.format(cwd), fg='green')
+
     # Initialize configuration file name and timer.
-    conf_file_name = '{}.json'.format(basename(getcwd()))
+    conf_file_name = '{}.json'.format(cwd)
     t0 = time()
 
+    # If clear option is on, remove all files and folders and exit.
+    if clear:
+        click.secho('Clearing directory of astro_reduce files and folders...',
+                    fg='green', nl=False)
+        if exists(conf_file_name):
+            remove(conf_file_name)
+        for folder in [OBJ, FLAT, DARK, RED, TMP]:
+            if exists(folder):
+                rmtree(folder, ignore_errors=True)
+        click.secho(' Done.', fg='green')
+        exit(0)
+
     # If setup option is on, set up the directory for reduction.
     if setup:
-        click.echo('Setting up for reduction.')
+        click.secho('Setting up for reduction:', fg='green')
         # Make sure the user image folders are there:
         if not (exists(UOBJ) and exists(UFLAT) and exists(UDARK)):
-            click.echo('E: Did not find folder `DARK`, `FLAT` or `ORIGINAL`\n'
-                       'E: containing the raw images to be reduced.\n'
-                       'E: Refer to the documentation for details.')
+            click.secho('E: Did not find folder `DARK`, `FLAT` or `ORIGINAL`\n'
+                        'E: containing the raw images to be reduced.\n'
+                        'E: Refer to the documentation for details.', fg='red')
             exit(1)
 
         # Initialize objects, exposure, filters lists, and working directories.
         objects = list()
         exposures = list()
         filters = list()
-        if exists(OBJ):
-            rmtree(OBJ, ignore_errors=True)
-        mkdir(OBJ)
-        if exists(DARK):
-            rmtree(DARK, ignore_errors=True)
-        mkdir(DARK)
-        if exists(FLAT):
-            rmtree(FLAT, ignore_errors=True)
-        mkdir(FLAT)
+        for folder in [OBJ, DARK, FLAT]:
+            if exists(folder):
+                rmtree(folder, ignore_errors=True)
+            mkdir(folder)
 
         # Open all images, retrieve exposures, filters, etc. and copy files to
         # astro_reduce working directories. This way the images are backed-up
         # at the same time.
         if verbose:
-            click.echo('Copying dark field images... ', nl=False)
+            click.secho('  Copying dark field images...', nl=False)
         for file in glob('{}/*.fit*'.format(UDARK)):
             exp, fn = dark_read_header(file)
             exposures.append(exp)
             copy(file, '{}/{}'.format(DARK, fn))
         if verbose:
-            click.echo('Done.')
+            click.secho('     Done.')
 
         if verbose:
-            click.echo('Copying flat field images... ', nl=False)
+            click.echo('  Copying flat field images...', nl=False)
         for file in glob('{}/*.fit*'.format(UFLAT)):
             fil, exp, fn = flat_read_header(file)
             exposures.append(exp)
             filters.append(fil)
             copy(file, '{}/{}'.format(FLAT, fn))
         if verbose:
-            click.echo('Done.')
+            click.echo('     Done.')
 
         if verbose:
-            click.echo('Copying object images... ', nl=False)
+            click.echo('  Copying object images...', nl=False)
         for file in glob('{}/*.fit*'.format(UOBJ)):
             obj, fil, exp, fn = obj_read_header(file)
             objects.append(obj)
@@ -276,29 +132,35 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
             exposures.append(exp)
             copy(file, '{}/{}'.format(OBJ, fn))
         if verbose:
-            click.echo('Done.')
+            click.echo('         Done.')
 
         # End up the setup by writing the configuration file.
-        click.echo('Writing configuration file `{}`.'.format(conf_file_name))
+        if verbose:
+            click.echo('  Writing configuration file `{}`.'.format(
+                                                                conf_file_name))
         write_conf_file(list(set(objects)), list(set(exposures)),
                         list(set(filters)), conf_file_name)
 
+        click.secho('Done.', fg='green')
+        exit(0)
+
     # Parse configuration file to obtain configuration dictionary.
     try:
-        click.echo('Parsing configuration file `{}`.'.format(conf_file_name))
+        click.secho('Parsing configuration file `{}`.'.format(conf_file_name),
+                    fg='green')
         with open(conf_file_name, 'r') as cfile:
             conf_dic = loads(cfile.read())
     except FileNotFoundError:
-        click.echo('E: Configuration file `{}` not found.\n'
-                   'E: If it is the first time you run astro_reduce in this\n'
-                   'E: directory, use the `--setup` option to setup the\n'
-                   'E: reduction and generate a configuration file.'
-                   ''.format(conf_file_name))
+        click.secho('E: Configuration file `{}` not found.\n'
+                    'E: If it is the first time you run astro_reduce in this\n'
+                    'E: directory, use the --setup option to setup the\n'
+                    'E: reduction and generate a configuration file.'
+                    ''.format(conf_file_name), fg='red')
         exit(1)
     except decoder.JSONDecodeError:
-        click.echo('E: Unable to parse configuration file `{}`.\n'
-                   'E: Fix by rerunning astro_reduce with the --setup option.'
-                   ''.format(conf_file_name))
+        click.secho('E: Unable to parse configuration file `{}`.\n'
+                    'E: Fix by rerunning astro_reduce with the --setup option.'
+                    ''.format(conf_file_name), fg='red')
         exit(1)
 
     # Obtain list of all object, dark, flat field file names.
@@ -314,9 +176,10 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
 
     # Check working directories are still there.
     if not (exists(OBJ) and exists(FLAT) and exists(DARK)):
-        click.echo('E: Seems like astro_reduce\'s working folders\n'
-                   'E: (those starting with `ar_`) were removed.\n'
-                   'E: Please rerun astro_reduce with the `--setup` option.')
+        click.secho('E: Seems like astro_reduce\'s working folders\n'
+                    'E: (those starting with `ar_`) were removed.\n'
+                    'E: Please rerun astro_reduce with the --setup option.',
+                    fg='red')
         exit(1)
 
     # Check all images are same size (if not we'll have a problem).
@@ -324,55 +187,56 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
                    glob('{}/*'.format(DARK))
                    + glob('{}/*'.format(FLAT))
                    + glob('{}/*'.format(OBJ))))) != 1:
-        click.echo('E: Seems like all image files don\'t have the same size.\n'
-                   'E: Please remove offending files and rerun astro_reduce\n'
-                   'E: with the `--setup` option.')
+        click.secho('E: Seems like all image files don\'t have the same size.\n'
+                    'E: Please remove offending files and rerun astro_reduce\n'
+                    'E: with the --setup option.', fg='red')
         exit(1)
 
     # Check if files exist.
     for key in object_files:
         if not object_files[key]:
-            click.echo('E: Did not find files for {} object.'.format(key))
+            click.secho('E: Did not find files for {} object.'.format(key),
+                        fg='red')
             exit(1)
     for key in dark_files:
         if not dark_files[key] and not interpolate:
             # If the interpolate option is off and there are some darks
             # missing, exit.
-            click.echo('E: Did not find dark field images for {}ms exposure.\n'
-                       'E: In order to interpolate the missing dark fields\n'
-                       'E: from the ones available, rerun using the\n'
-                       'E: `--interpolate` option.'.format(key))
+            click.secho('E: Did not find dark field images for {}ms exposure.\n'
+                        'E: In order to interpolate the missing dark fields\n'
+                        'E: from the ones available, rerun using the\n'
+                        'E: --interpolate option.'.format(key), fg='red')
             exit(1)
     for key in flat_files:
         if not flat_files[key]:
-            click.echo('E: No flat field images for {} filter.'.format(key))
+            click.secho('E: No flat field images for {} filter.'.format(key),
+                        fg='red')
             exit(1)
 
     # Report all files found.
     reg = compile(r'_[a-z0-9]*\.')
-    tstring = '****** {:25} ******'
-    sstring = '    {:8}: {}'
     if verbose:
         handy = lambda x: reg.sub('_*.', basename(x))
-        click.echo('Files found:')
-        click.echo(tstring.format(' Objects (`{}`) '.format(OBJ)))
+        click.secho('Files found:', fg='green')
+        click.secho('  Objects (`{}`):'.format(OBJ), fg='blue')
         for obj in conf_dic['objects']:
             uniq_names = set(map(handy, object_files[obj]))
-            click.echo(sstring.format(obj, uniq_names))
+            click.echo('    {:10}: {}'.format(obj, uniq_names))
 
-        click.echo(tstring.format(' Dark fields (`{}`) '.format(DARK)))
+        click.secho('  Dark fields (`{}`):'.format(DARK), fg='blue')
         for exp in conf_dic['exposures']:
             uniq_names = set(map(handy, dark_files[exp])) or None
-            click.echo(sstring.format('{}'.format(exp), uniq_names))
+            click.echo('    {:10}: {}'.format('{}ms'.format(exp), uniq_names))
 
-        click.echo(tstring.format(' Flat fields (`{}`) '.format(FLAT)))
+        click.secho('  Flat fields (`{}`):'.format(FLAT), fg='blue')
         for filt in conf_dic['filters']:
             uniq_names = set(map(handy, flat_files[filt]))
-            click.echo(sstring.format(filt, uniq_names))
+            click.echo('    {:10}: {}'.format(filt, uniq_names))
 
     # STEP 0: Create directory for tmp and reduced images if not existent.
     if verbose:
-        click.echo('Creating folders to hold reduced and intermediate images.')
+        click.secho('Creating folders to hold reduced and intermediate images.',
+                    fg='green')
     if exists(RED):
         rmtree(RED, ignore_errors=True)
     mkdir(RED)
@@ -382,12 +246,12 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
 
     # STEP 1: Write the master dark files (medians of darks)
     # for each available exposure.
-    click.echo('Writing master dark images.')
+    click.secho('Writing master dark images:', fg='green')
     all_exposures = conf_dic['exposures']
     available_exposures = [exp for exp in dark_files if dark_files[exp]]
     for exp in available_exposures:
         if verbose:
-            click.echo('    {}ms... '.format(exp), nl=False)
+            click.echo('    {:14} '.format('{}ms...'.format(exp)), nl=False)
         mdark_data = np.median([fits.getdata(_) for _ in dark_files[exp]],
                                axis=0)
         mdark_header = fits.getheader(dark_files[exp][0])
@@ -410,7 +274,8 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
     # such that (missing_dark) = a * (exposure_time) + b.
     # Exit if there are no darks at all.
     if not available_exposures:
-        click.echo('E: There are no dark files at all! Cannot interpolate...')
+        click.secho('E: There are no dark files at all! Cannot interpolate...',
+                    fg='red')
         exit(1)
 
     if len(available_exposures) == 1:
@@ -439,10 +304,10 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
         b = my - mx * a
 
     # Write all the missing master darks!
-    click.echo('Interpolating missing master dark images.')
+    click.secho('Interpolating missing master dark images:', fg='green')
     for exp in list(set(all_exposures) - set(available_exposures)):
         if verbose:
-            click.echo('    {}ms... '.format(exp), nl=False)
+            click.echo('    {:14} '.format('{}ms...'.format(exp)), nl=False)
         new_mdark_data = float(exp) * a + b
 
         # Write fits file and header.
@@ -458,12 +323,12 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
 
     # STEP 2: Write master transmission files for each filter:
     # mtrans = median(normalized(flat - dark of same exposure)).
-    click.echo('Calculating master transmission images.')
+    click.secho('Calculating master transmission images:', fg='green')
     # Handy function to extract exposure from flat file name.
     fexp = lambda fname: fname.split('.fit')[0].split('_')[-2]
     for filt in flat_files:
         if verbose:
-            click.echo('    {}... '.format(filt), nl=False)
+            click.echo('    {:12}   '.format('{}...'.format(filt)), nl=False)
 
         # Calculate normalized flats.
         normalized_flats = list()
@@ -489,10 +354,10 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
 
     # STEP 3: Reduce all the object images with corresponding filter mtrans
     # and exposure mdark.
-    click.echo('Writing auxiliary object images.')
+    click.secho('Writing auxiliary object images:', fg='green')
     for obj in object_files:
         if verbose:
-            click.echo('    {}... '.format(obj), nl=False)
+            click.echo('    {:12}   '.format('{}...'.format(obj)), nl=False)
         for fname in object_files[obj]:
             bfname = basename(fname)
             filt, exp = fname_bits(bfname)
@@ -506,7 +371,7 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
             aux_header = fits.getheader(fname)
 
             # Write fits file and header.
-            nname = '{}/{}_{}.fits'.format(TMP, bfname.split(".fit")[0], AUX)
+            nname = '{}/{}_{}.fits'.format(TMP, bfname.split('.fit')[0], AUX)
             fits.writeto(nname, aux_data, aux_header, overwrite=True)
             fits.setval(nname, 'FILTER', value=filt)
             fits.setval(nname, 'IMAGETYP', value='Light Frame')
@@ -518,10 +383,10 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
 
     # STEP 4: For all objects realign and median the aux images.
     # You are left with one image per object per filter per exposure.
-    click.echo('Realigning object images.')
+    click.secho('Realigning object images:', fg='green')
     for obj in object_files:
         if verbose:
-            click.echo('    {}:'.format(obj))
+            click.secho('  {}:'.format(obj), fg='blue')
         # Group all the object files by *tag*, i.e. by filter, exposure.
         name_tag_hash = [(basename(fname),
                          '{}'.format(fname_bits(basename(fname))))
@@ -536,7 +401,8 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
             # e.g., the first name in the list.)
             f, e = fname_bits(names_per_tag[tag][0])
             if verbose:
-                click.echo('      {}:{}... '.format(f, e))
+                click.echo('    {:23}'.format('{}:{}ms...'.format(f, e)),
+                           nl=False)
             # Calculate aligned and medianed image
             # from all images with same tag.
             aux_files = glob('{}/{}_{}_{}_*_{}.fits'.format(TMP, obj, f,
@@ -555,24 +421,26 @@ def cli(setup, interpolate, verbose, tmppng, redpng):
             fits.setval(nname, 'EXPOSURE', value=float(e) / 1000., comment=hc)
             fits.setval(nname, 'OBJECT', value=obj)
             if verbose:
-                click.echo(' ' * (11 + len(f) + len(e))
-                           + 'Done ({} images).'.format(len(aux_files)))
+                click.echo('       Done ({} images).'.format(len(aux_files)))
 
     # STEP 5: If options redpng or tmppng are on, write
     # PNG versions of all the tmp and reduced images.
     if redpng:
-        click.echo('Writing PNG versions of reduced images... ', nl=False)
+        click.secho('Writing PNG versions of reduced images... ',
+                    fg='green', nl=False)
         for ffile in glob('{}/*.fits'.format(RED)):
             write_png(ffile, plt)
-        click.echo('Done.')
+        click.secho('Done.', fg='green')
 
     if tmppng:
-        click.echo('Writing PNG versions of intermediate images... ', nl=False)
+        click.secho('Writing PNG versions of intermediate images... ',
+                    fg='green', nl=False)
         for ffile in glob('{}/*.fits'.format(TMP)):
             write_png(ffile, plt)
-        click.echo('Done.')
+        click.secho('Done.', fg='green')
 
     # Report execution time.
     t1 = time()
-    click.echo('All done. ({})'.format(timedelta(seconds=int(t1 - t0))))
+    click.secho('\nAll done. ({})'.format(timedelta(seconds=int(t1 - t0))),
+                fg='green')
     # ALL DONE.
